package cz.fidentis.analyst.visitors.kdtree;

import cz.fidentis.analyst.kdtree.KdTree;
import cz.fidentis.analyst.kdtree.KdTreeVisitor;
import cz.fidentis.analyst.mesh.core.CornerTableRow;
import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.mesh.core.MeshFacetImpl;
import cz.fidentis.analyst.mesh.core.MeshPoint;
import cz.fidentis.analyst.mesh.core.MeshPointImpl;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import javax.vecmath.Vector3d;
import static org.junit.jupiter.api.Assertions.assertEquals;
import org.junit.jupiter.api.Test;

public class KdTreeDistanceToVerticesTest {
    
    protected MeshFacet getTrivialFacet(double offset, double size) {
        MeshFacet facet = new MeshFacetImpl();
        facet.addVertex(new MeshPointImpl(new Vector3d(0, 0, offset), new Vector3d(0, 0, 1), new Vector3d()));
        facet.addVertex(new MeshPointImpl(new Vector3d(size, 0, offset), new Vector3d(0, 0, 1), new Vector3d()));
        facet.addVertex(new MeshPointImpl(new Vector3d(0, size, offset), new Vector3d(0, 0, 1), new Vector3d()));

        facet.getCornerTable().addRow(new CornerTableRow(0, -1));
        facet.getCornerTable().addRow(new CornerTableRow(1, -1));
        facet.getCornerTable().addRow(new CornerTableRow(2, -1));

        return facet;
    }
    
    protected void distTest(double expectedDist, KdTreeDistanceToVertices vis) {
        KdTree tree = new KdTree(getTrivialFacet(1.5, 1));
        tree.accept(vis);
        assertEquals(expectedDist, vis.getDistance());
    }
    
    protected void concurrentDistTest(double expectedDist, KdTreeDistanceToVertices vis) throws InterruptedException, ExecutionException {
        final int threads = Runtime.getRuntime().availableProcessors();
        final ExecutorService executor = Executors.newFixedThreadPool(threads);
        final List<Future<KdTreeVisitor>> results = new LinkedList<>();
        
        final KdTree tree = new KdTree(getTrivialFacet(1.5, 1));
        tree.accept(vis);
        
        final Future<KdTreeVisitor> result = executor.submit(vis);
        results.add(result);
        results.get(0).get();
        
        assertEquals(expectedDist, vis.getDistance());
    }
    
    @Test
    public void distTest() {
        KdTreeDistanceToVertices vis = new KdTreeDistanceToVertices(new Vector3d(0,0,0), false); // sequentially
        distTest(1.5, vis);
    }

    @Test
    public void exactMatchTest() {
        MeshPoint point = new MeshPointImpl(new Vector3d(0, 0, 1.5), new Vector3d(0, 0, 1), new Vector3d());
        KdTreeDistanceToVertices vis = new KdTreeDistanceToVertices(new Vector3d(0, 0, 1.5), false); // relative dist, sequentially
        distTest(0, vis);
    }
    
    @Test
    public void concurrencyTest() throws InterruptedException, ExecutionException {
        KdTreeDistanceToVertices vis = new KdTreeDistanceToVertices(new Vector3d(0,0,0), true); // absolute dist, concurrently
        concurrentDistTest(1.5, vis);
    }
}
